@@ -2,7 +2,7 @@
 #'
 #'Fits a linear mixed effects model for every protein (by OlinkID) in every panel, using lmerTest::lmer and stats::anova.
 #'The function handles both factor and numerical variables and/or covariates. \cr\cr
-#'Samples that have no variable information or missing factor levels are automatically removed from the analysis (specified in a messsage if verbose = TRUE).
+#'Samples that have no variable information or missing factor levels are automatically removed from the analysis (specified in a message if verbose = TRUE).
 #'Character columns in the input dataframe are automatically converted to factors (specified in a message if verbose = TRUE).
 #'Numerical variables are not converted to factors.
 #'If a numerical variable is to be used as a factor, this conversion needs to be done on the dataframe before the function call. \cr\cr
@@ -13,7 +13,7 @@
 #' \item c('A:B', 'B') or c('A:B', 'A')
 #'}
 #'Inference is specified in a message if verbose = TRUE. \cr
-#'For covariates, crossed analyses need to be specified explicity, i.e. two main effects will not be expaned with a c('A','B') notation. Main effects present in the variable takes precedence. \cr
+#'For covariates, crossed analyses need to be specified explicitly, i.e. two main effects will not be expanded with a c('A','B') notation. Main effects present in the variable takes precedence. \cr
 #'The random variable only takes main effect(s). \cr
 #'The formula notation of the final model is specified in a message if verbose = TRUE. \cr\cr
 #'Output p-values are adjusted by stats::p.adjust according to the Benjamini-Hochberg method (“fdr”).
@@ -25,8 +25,8 @@
 #' Also takes ':' or '*' notation.
 #' @param outcome Character. The dependent variable. Default: NPX.
 #' @param random Single character value or character array.
-#' @param covariates Single character value or character array. Default: NULL.
-#' Covariates to include. Takes ':' or '*' notation. Crossed analysis will not be inferred from main effects.
+#' @param covariates Single character value or character array. Default: NULL.Covariates to include. Takes ':' or '*' notation. Crossed analysis will not be inferred from main effects.
+#' @param model_formula (optional) Symbolic description of the model to be fitted in standard formula notation (e.g. "NPX~A*B + (1|ID)"). If provided, this will override the \code{outcome}, \code{variable} and \code{covariates} arguments. Can be a string or of class \code{stats::formula()}.
 #' @param return.covariates Boolean. Default: False. Returns results for the covariates. Note: Adjusted p-values will be NA for the covariates.
 #' @param verbose Boolean. Default: True. If information about removed samples, factor conversion and final model formula is to be printed to the console.
 #'
@@ -60,7 +60,7 @@
 #' @importFrom dplyr n filter group_by summarise ungroup pull distinct group_modify mutate select all_of
 #' @importFrom lmerTest lmer
 #' @importFrom rlang ensym
-#' @importFrom stringr str_detect
+#' @importFrom stringr str_detect str_extract
 #' @importFrom generics tidy
 #' @importFrom lme4 lmerControl
 
@@ -69,10 +69,32 @@ olink_lmer <- function(df,
                        outcome="NPX",
                        random,
                        covariates = NULL,
+                       model_formula,
                        return.covariates = FALSE,
                        verbose = TRUE
 ) {
 
+  if(!missing(model_formula)){
+    if("formula" %in% class(model_formula)) model_formula <- deparse(model_formula) #Convert to string if is formula
+    tryCatch(as.formula(model_formula),error=function(e) stop(paste0(model_formula," is not a recognized formula."))) #If cannot be coerced into formula, error
+    #If variable, random or covariates were included, message that they will not be used
+    if(!missing(variable) | !is.null(covariates) | !missing(random)) message("model_formula overriding variable, covariate and random arguments.")
+    #Parse formula so checks on the  variable, outcome and random objects can continue as usual
+    model_formula <- gsub(" ","",model_formula)
+    #Random portion of formula
+    splt_random <- stringr::str_extract(string = model_formula, pattern = "(?<=\\().*(?=\\))")
+    splt_random <- strsplit(splt_random,"\\+|~|\\*|:|\\|")[[1]]
+    if(any(grepl("-1|1",splt_random))) splt_random <- splt_random[-grep("-1|1",splt_random)]
+    random <- splt_random
+    #Fixed effects portion
+    splt_form <- gsub("\\s*\\([^\\)]+\\)","",model_formula)
+    splt_form <- strsplit(splt_form,c("\\+|~|\\*|:"))[[1]]
+    if("-1" %in% splt_form) splt_form <- splt_form[-which(splt_form=="-1")]
+    outcome <- splt_form[1]
+    variable <- splt_form[-1]
+    covariates <- NULL
+  }
+  
   if(missing(df) | missing(variable) | missing(random)){
     stop('The df and variable and random arguments need to be specified.')
   }
@@ -201,20 +223,24 @@ olink_lmer <- function(df,
 
     }
 
-    if(!is.null(covariates)){
-      formula_string <- paste0(outcome, "~",
-                               paste(variable,collapse="*"),
-                               "+",
-                               paste(covariates, sep = '', collapse = '+'),
-                               "+",
-                               paste(paste0("(1|",random,")"),collapse="+"))
-    }else{
-
-      formula_string <- paste0(outcome, "~", paste(variable,collapse="*"),
-                               "+",
-                               paste(paste0("(1|",random,")"),collapse="+"))
+    if(missing(model_formula)){
+      if(!is.null(covariates)){
+        formula_string <- paste0(outcome, "~",
+                                 paste(variable,collapse="*"),
+                                 "+",
+                                 paste(covariates, sep = '', collapse = '+'),
+                                 "+",
+                                 paste(paste0("(1|",random,")"),collapse="+"))
+      }else{
+        
+        formula_string <- paste0(outcome, "~", paste(variable,collapse="*"),
+                                 "+",
+                                 paste(paste0("(1|",random,")"),collapse="+"))
+      }
+    } else if(!missing(model_formula)){
+      formula_string <- model_formula
     }
-
+    
     #Get factors
     fact.vars <- sapply(variable_testers, function(x) is.factor(df[[x]]))
     fact.vars <- names(fact.vars)[fact.vars]
@@ -255,7 +281,7 @@ olink_lmer <- function(df,
       dplyr::filter(!(OlinkID %in% all_nas)) %>%
       dplyr::filter(!(OlinkID %in% nas_in_var)) %>%
       dplyr::group_by(Assay, OlinkID, UniProt, Panel) %>%
-      dplyr::group_modify(~generics::tidy(stats::anova(single_lmer(data=.x, formula_string = formula_string)))) %>%
+      dplyr::group_modify(~generics::tidy(stats::anova(single_lmer(data=.x, formula_string = formula_string),type="III",ddf="Satterthwaite"))) %>%
       dplyr::ungroup() %>%
       dplyr::mutate(covariates = term %in% covariate_filter_string) %>%
       dplyr::group_by(covariates) %>%
@@ -301,7 +327,7 @@ single_lmer <- function(data, formula_string){
   )
 
 
-  if(class(out.model)=="lmerModLmerTest"){
+  if(inherits(out.model,"lmerModLmerTest")){
     return(out.model)
   } else{
     stop("Convergence issue not caught by single_lmer")
@@ -321,16 +347,18 @@ single_lmer <- function(data, formula_string){
 #'
 #' @param df NPX data frame in long format with at least protein name (Assay), OlinkID, UniProt, 1-2 variables with at least 2 levels and subject ID.
 #' @param olinkid_list Character vector of OlinkID's on which to perform post hoc analysis. If not specified, all assays in df are used.
-#' @param effect Term on which to perform post-hoc. Character vector. Must be subset of or identical to variable.
-#' @param outcome Character. The dependent variable. Default: NPX.
-#' @param random Single character value or character array.
-#' @param covariates Single character value or character array. Default: NULL.
-#' Covariates to include. Takes ':' or '*' notation. Crossed analysis will not be inferred from main effects.
-#' @param mean_return Boolean. If true, returns the mean of each factor level rather than the difference in means (default). Note that no p-value is returned for mean_return = TRUE and no adjustment is performed.
-#' @param verbose Boolean. Deafult: True. If information about removed samples, factor conversion and final model formula is to be printed to the console.
 #' @param variable Single character value or character array.
 #' Variable(s) to test. If length > 1, the included variable names will be used in crossed analyses .
 #' Also takes ':' or '*' notation.
+#' @param covariates Single character value or character array. Default: NULL. Covariates to include. Takes ':' or '*' notation. Crossed analysis will not be inferred from main effects.
+#' @param random Single character value or character array.
+#' @param outcome Character. The dependent variable. Default: NPX.
+#' @param model_formula (optional) Symbolic description of the model to be fitted in standard formula notation (e.g. "NPX~A*B + (1|ID)"). If provided, this will override the \code{outcome}, \code{variable} and \code{covariates} arguments. Can be a string or of class \code{stats::formula()}.
+#' @param effect Term on which to perform post-hoc. Character vector. Must be subset of or identical to variable.
+#' @param effect_formula (optional) A character vector specifying the names of the predictors over which estimated marginal means are desired as defined in the \code{emmeans} package. May also be a formula. If provided, this will override the \code{effect} argument. See \code{?emmeans::emmeans()} for more information.
+#' @param mean_return Boolean. If true, returns the mean of each factor level rather than the difference in means (default). Note that no p-value is returned for mean_return = TRUE and no adjustment is performed.
+#' @param post_hoc_padjust_method P-value adjustment method to use for post-hoc comparisons within an assay. Options include \code{tukey}, \code{sidak}, \code{bonferroni} and \code{none}.
+#' @param verbose Boolean. Default: True. If information about removed samples, factor conversion and final model formula is to be printed to the console.
 #'
 #' @return A "tibble" containing the results of the pairwise comparisons between given variable levels for proteins specified in olinkid_list (or full df).
 #' Columns include:
@@ -344,7 +372,7 @@ single_lmer <- function(data, formula_string){
 #'  \item{estimate:} "numeric" difference in mean NPX between groups
 #'  \item{conf.low:} "numeric" confidence interval for the mean (lower end)
 #'  \item{conf.high:} "numeric" confidence interval for the mean (upper end)
-#'  \item{Adjusted_pval:} "numeric" adjusted p-value for the test (Benjamini&Hochberg)
+#'  \item{Adjusted_pval:} "numeric" adjusted p-value for the test 
 #'  \item{Threshold:} "character" if adjusted p-value is significant or not (< 0.05)
 #' }
 #' 
@@ -369,7 +397,18 @@ single_lmer <- function(data, formula_string){
 #'                                            variable=c("Time", 'Treatment'),
 #'                                            effect = 'Time:Treatment',
 #'                                            random = 'Subject',
-#'                                            verbose = TRUE)}
+#'                                            verbose = TRUE)
+#'                                            
+#' #Estimate treated vs untreated at each timepoint
+#' 
+#' 
+#'results_lmer_posthoc <- olink_lmer_posthoc(df = npx_data1,
+#'                                            olinkid_list = assay_list,
+#'                                            model_formula = "NPX~Time*Treatment+(1|Subject)",
+#'                                            effect_formula = "pairwise~Treatment|Time",
+#'                                            verbose = TRUE)
+#' }
+#'                                                                                     
 #' @importFrom magrittr %>%
 #' @importFrom dplyr filter group_by summarise ungroup pull distinct group_modify mutate select rename arrange
 #' @importFrom lmerTest lmer
@@ -377,16 +416,55 @@ single_lmer <- function(data, formula_string){
 #' @importFrom emmeans emmeans
 
 olink_lmer_posthoc <- function(df,
-                               variable,
                                olinkid_list = NULL,
-                               effect,
+                               variable,
                                outcome="NPX",
                                random,
+                               model_formula,
+                               effect,
+                               effect_formula,
                                covariates = NULL,
                                mean_return = FALSE,
+                               post_hoc_padjust_method="tukey",
                                verbose = TRUE
 ){
 
+  if(!missing(model_formula)){
+    if("formula" %in% class(model_formula)) model_formula <- deparse(model_formula) #Convert to string if is formula
+    tryCatch(as.formula(model_formula),error=function(e) stop(paste0(model_formula," is not a recognized formula."))) #If cannot be coerced into formula, error
+    #If variable, random or covariates were included, message that they will not be used
+    if(!missing(variable) | !is.null(covariates) | !missing(random)) message("model_formula overriding variable, covariate and random arguments.")
+    #Parse formula so checks on the  variable, outcome and random objects can continue as usual
+    model_formula <- gsub(" ","",model_formula)
+    #Random portion of formula
+    splt_random <- stringr::str_extract(string = model_formula, pattern = "(?<=\\().*(?=\\))")
+    splt_random <- strsplit(splt_random,"\\+|~|\\*|:|\\|")[[1]]
+    if(any(grepl("-1|1",splt_random))) splt_random <- splt_random[-grep("-1|1",splt_random)]
+    random <- splt_random
+    #Fixed effects portion
+    splt_form <- gsub("\\s*\\([^\\)]+\\)","",model_formula)
+    splt_form <- strsplit(splt_form,c("\\+|~|\\*|:"))[[1]]
+    if("-1" %in% splt_form) splt_form <- splt_form[-which(splt_form=="-1")]
+    outcome <- splt_form[1]
+    variable <- splt_form[-1]
+    covariates <- NULL
+  }
+  
+  if(!missing(effect_formula)){
+    
+    if(length(effect_formula)==1){
+      #Parse effect formula so the check on the effect object can continue as usual
+      if(!missing(effect)) message("effect_formula overriding effect argument.")
+      if("formula" %in% class(effect_formula)) effect_formula <- deparse(effect_formula)
+      splt_effect <- effect_formula
+      if(grepl("~",splt_effect)) splt_effect <- strsplit(splt_effect,"~")[[1]][2] #Pull out variables from right hand side of formula. e.g. pairwise~A+B|C = "A+B|C"
+      if(grepl("\\||+|\\*",splt_effect)) splt_effect <- strsplit(splt_effect,"\\||\\+|\\*")[[1]] #Split rhs of formula into vector of variables. e.g. "A+B|C"=c("A","B","C")
+
+      effect <- splt_effect
+    } else{
+      stop("Unrecognized effect formula. Should be a character string of length 1. If listing in the form c('A','B'), use the effects argument.")
+    }
+  }
 
   if(missing(df) | missing(variable) | missing(effect) | missing(random)){
     stop('The df, variable, random and effect arguments need to be specified.')
@@ -452,18 +530,28 @@ olink_lmer_posthoc <- function(df,
     }
 
 
-    if(!is.null(covariates)){
-      formula_string <- paste0(outcome, "~",
-                               paste(variable,collapse="*"),
-                               "+",
-                               paste(covariates, sep = '', collapse = '+'),
-                               "+",
-                               paste(paste0("(1|",random,")"),collapse="+"))
-    }else{
-
-      formula_string <- paste0(outcome, "~", paste(variable,collapse="*"),
-                               "+",
-                               paste(paste0("(1|",random,")"),collapse="+"))
+    if(missing(model_formula)){
+      if(!is.null(covariates)){
+        formula_string <- paste0(outcome, "~",
+                                 paste(variable,collapse="*"),
+                                 "+",
+                                 paste(covariates, sep = '', collapse = '+'),
+                                 "+",
+                                 paste(paste0("(1|",random,")"),collapse="+"))
+      }else{
+        
+        formula_string <- paste0(outcome, "~", paste(variable,collapse="*"),
+                                 "+",
+                                 paste(paste0("(1|",random,")"),collapse="+"))
+      }
+    } else if(!missing(model_formula)){
+      formula_string <- model_formula
+    }
+    
+    if(!missing(effect_formula)){
+      e_form <- effect_formula
+    } else if(missing(effect_formula)){
+      e_form <- paste0("pairwise~", paste(effect,collapse="+"))
     }
 
     #Print verbose message
@@ -497,8 +585,9 @@ olink_lmer_posthoc <- function(df,
       dplyr::group_by(Assay, OlinkID, UniProt, Panel) %>%
       dplyr::group_modify(~single_posthoc(data = .x,
                                    formula_string=formula_string,
-                                   effect = effect,
-                                   mean_return = mean_return)) %>%
+                                   effect = e_form,
+                                   mean_return = mean_return,
+                                   padjust_method=post_hoc_padjust_method)) %>%
       dplyr::ungroup() %>%
       dplyr::mutate(term=paste(effect,collapse=":"))  %>%
       dplyr::select(Assay, OlinkID, UniProt, Panel, term, everything())
@@ -519,34 +608,39 @@ olink_lmer_posthoc <- function(df,
 
 }
 
-single_posthoc <- function(data, formula_string, effect, mean_return){
+single_posthoc <- function(data, formula_string, effect, mean_return, padjust_method="tukey"){
+  
+  if(!is.character(effect)) stop("effect must be a character string.")
 
   the_model <- emmeans::emmeans(single_lmer(data, formula_string),
-                                specs=as.formula(paste0("pairwise~", paste(effect,collapse="+"))),
+                                specs=as.formula(effect), #effect must be string to be converted to as.formula
                                 cov.reduce = function(x) round(c(mean(x),mean(x)+sd(x)),4),
                                 lmer.df="satterthwaite",
                                 infer = c(TRUE, TRUE),
-                                adjust = 'tukey')
+                                adjust = padjust_method)
 
   if(mean_return){
-    tmp <- unique(unlist(strsplit(effect,":")))
+    # tmp <- unique(unlist(strsplit(effect,":")))
     return(as_tibble(the_model$emmeans) %>%
              dplyr::rename(conf.low=lower.CL,
                     conf.high=upper.CL) %>%
-             dplyr::select(dplyr::all_of(c(tmp, "emmean", "conf.low", "conf.high")))
+             dplyr::select(-SE,-df,-t.ratio,-p.value)
     )
 
   }else{
-    return(as_tibble(the_model$contrasts) %>%
-             dplyr::rename(Adjusted_pval = p.value) %>%
-             dplyr::mutate(Threshold = if_else(Adjusted_pval < 0.05,
+    out_df <- as_tibble(the_model$contrasts) %>%
+      dplyr::rename(Adjusted_pval = p.value) %>%
+      dplyr::mutate(Threshold = if_else(Adjusted_pval < 0.05,
                                         'Significant',
                                         'Non-significant')) %>%
-             dplyr::rename(conf.low=lower.CL,
+      dplyr::rename(conf.low=lower.CL,
                     conf.high=upper.CL) %>%
-             dplyr::select(contrast, estimate, conf.low, conf.high, Adjusted_pval,Threshold) %>%
-             dplyr::arrange(Adjusted_pval)
-    )
+      dplyr::select(-SE,-df,-t.ratio) %>% 
+      dplyr::arrange(Adjusted_pval)
+    
+    if(padjust_method=="none") out_df <- out_df %>% rename(pvalue=Adjusted_pval)
+    
+    return(out_df)
 
   }
 
